import numpy as np
import scipy.linalg as nla

# L'exemple du TD
A = np.array ([[0,-20,-14],[3,27,-4],[4,11,-2]],dtype=np.float64,order='F')
b = np.array ([30,86,48],dtype=np.float64)
print ("A =")
print (A)
print ("b = ", b)

# La matrice A est modifiée.
# Dans la partie en haut à droite, on trouve R.
# On remarque que la fonction ne cherche pas à avoir des éléments 
#   diagonaux positifs. Sous la diagonale, il y a une partie des 
#   vecteurs v qui codent les matrices de réflexion. 

# Le vecteur b est modifié. Il reçoit Q^T . b.
# Comment expliquer les paramètres ?
#   En Python, on ne distingue pas vecteur ligne / vecteur colonne.
#   Donc, au lieu de calculer Q^T . b, on calcule (Q^T . b)^T = b^T . Q
#   Du point de vue Python, c'est pareil puisque b = b^T (c'est un vecteur).
#   Et il suffit donc de multiplier à droite par Q.

nla.qr_multiply (A, b, mode='right', overwrite_a=True, overwrite_c=True)
print ("Après qr_multiply on a : A =")
print (A)
print ("b =", b)

# On résout R . x = Q^T . b (résultat dans b)
# Q^T . b est stocké dans b
# R est dans la partie supérieure de A et ne contient pas 
#   nécessairement de 1 sur la diagonale.

nla.solve_triangular (A, b, lower=False, unit_diagonal=False, overwrite_b=True)

# Bingo
print ("b =", b)

